@@ -212,7 +212,7 @@ void programcmd(unsigned char seqnum)
                 msg_buf[1] = STATUS_CMD_FAILED;
                 break;
 
-                case CMD_ENTER_PROGMODE_ISP:
+                case CMD_ENTER_PROGMODE_ISP: // 0x10
                 // The syntax of this command is as follows:
                 // 0: Command ID 1 byte, CMD_ENTER_ PROGMODE_ISP
                 // 1: timeout 1 byte, Command time-out (in ms)
@@ -237,16 +237,17 @@ void programcmd(unsigned char seqnum)
                 //connect to the target
                 i=0;
                 // limit the loops:
-                if (msg_buf[4]> 32){
-                        msg_buf[4]=32;
+                if (msg_buf[4]> 48){
+                        msg_buf[4]=48;
                 }
-                if (msg_buf[5] < 3){
-                        // minimum delay
-                        msg_buf[5]=3;
+                if (msg_buf[5] < 1){
+                        // minimum byteDelay
+                        msg_buf[5]=1;
                 }
                 while(i<msg_buf[4]){//synchLoops
                         wd_kick();
                         LED_ON; // blink during init
+                        delay_ms(msg_buf[3]); //cmdexeDelay
                         i++;
                         spi_mastertransmit(msg_buf[8]);//cmd1
                         delay_ms(msg_buf[5]); //byteDelay
@@ -256,7 +257,7 @@ void programcmd(unsigned char seqnum)
                         tmp=spi_mastertransmit(msg_buf[10]);//cmd3
                         delay_ms(msg_buf[5]); //byteDelay
                         tmp2=spi_mastertransmit(msg_buf[11]);//cmd4
-                        delay_ms(msg_buf[5]); //byteDelay
+                        //
                         //7=pollIndex, 6=pollValue
                         if(msg_buf[7]==3 && tmp==msg_buf[6]) {
                                 msg_buf[1] = STATUS_CMD_OK;
@@ -268,11 +269,23 @@ void programcmd(unsigned char seqnum)
                         if(msg_buf[7]==0) { //pollIndex
                                 msg_buf[1] = STATUS_CMD_OK;
                         }
-                        // causes problems with MULTI spi_clock_pulse();
-                        if(msg_buf[1] == STATUS_CMD_OK)
-                        {
+                        if(msg_buf[1] == STATUS_CMD_OK ) {
                                 LED_ON;
                                 i=msg_buf[4];// end loop
+                        }else{
+                                // new try, see e.g chapeter "Serial download" in
+                                // data sheet.
+                                // in atmega8 it says: give positive reset pulse 
+                                //                     and try again
+                                // in at90s2313 it says: give positive sck pulse 
+                                //                     and try again
+                                // One of those datasheets must be wrong!?
+                                // Guido thinks that spi_sck_pulse is correct
+                                // and will also work for atmega8 given that
+                                // the number of synchLoops is at least 32.
+                                //spi_reset_pulse();
+                                spi_sck_pulse();
+                                delay_ms(20); 
                         }
                 }
                 break;
@@ -321,13 +334,13 @@ void programcmd(unsigned char seqnum)
                 // msg_buf[n+10] Data
                 poll_address=0;
 		ci=150;
-                // set a minimum delay
-                if (msg_buf[4] < 3){
-                        msg_buf[4]=3;
+                // set a minimum timed delay
+                if (msg_buf[4] < 4){
+                        msg_buf[4]=4;
                 }
                 // set a max delay
-                if (msg_buf[4] > 24){
-                        msg_buf[4]=24;
+                if (msg_buf[4] > 32){
+                        msg_buf[4]=32;
                 }
 		saddress=address; // previous address, start address 
 		nbytes = ((unsigned int)msg_buf[1])<<8;
@@ -373,7 +386,7 @@ void programcmd(unsigned char seqnum)
                                 wd_kick();
                                 if (!addressing_is_word){
                                         // eeprom writing, eeprom needs more time
-                                        delay_ms(1);
+                                        delay_ms(2);
                                 }
                                 //check the different polling mode methods
                                 if(msg_buf[3]& 0x04) {
@@ -582,6 +595,7 @@ void programcmd(unsigned char seqnum)
                 cj=0; 
                 ci=0;
                 for (cj=0; cj<msg_buf[1]; cj++) {
+			delay_ms(5);
                         if (cj >= tmp2 && ci <tmp){
                                 // store answer starting from msg_buf[2]
                                 msg_buf[ci+2]=spi_mastertransmit(msg_buf[cj+4]);
@@ -589,7 +603,6 @@ void programcmd(unsigned char seqnum)
                         }else{
                                 spi_mastertransmit(msg_buf[cj+4]);
                         }
-			delay_ms(5);
                 }
                 // padd with zero:
                 while(ci<tmp){
@@ -613,29 +626,35 @@ void programcmd(unsigned char seqnum)
         
 }
 
-// Read a max of 2 digits from the serial line
-// return value of -1 means no change (user just hit return)
-signed char terminalmode_readnum(unsigned char chr_nl){
+// Read a max of 2 hex digits from the serial line
+// return value of 0xff means no change (user just hit return)
+unsigned char terminalmode_readnum(unsigned char chr_nl){
         unsigned char ch;
         unsigned char lineendcnt=1;
         unsigned char digits=2; // you can enter max 2 digits
-        signed char rval=-1;
+        unsigned char rval=0xff;
 	if (chr_nl){
 		// we expect \r\n as line end
 		lineendcnt=2;
 	}
 	ch=uart_getchar(1);
-	while(ch != '\r' && ch !='\n' && digits){
-		if ((ch >= '0') && (ch <= '9')){
-			if (rval==-1){
-				rval=0;
-			}
-			uart_sendchar(ch); // echo number back to terminal
-			rval=rval*10 + (ch  - (unsigned char)'0');
-			digits--;
-		}
-		ch=uart_getchar(1);
-	}
+        while(ch != '\r' && ch !='\n' && digits){
+                if (((ch >= '0') && (ch <= '9'))||((ch >= 'A') && (ch <= 'F'))||((ch >= 'a') && (ch <= 'f'))){
+                        if (rval==0xff){
+                                rval=0;
+                        }
+                        uart_sendchar(ch); // echo number back to terminal
+                        if((ch >= '0') && (ch <= '9')) {
+                            rval=rval*16 + (ch  - (unsigned char)'0');
+                        } else if ((ch >= 'A') && (ch <= 'F')){
+                            rval=rval*16 + (ch  - (unsigned char)'A' + 10);
+                        } else {
+                            rval=rval*16 + (ch  - (unsigned char)'a' + 10);
+                        }
+                        digits--;
+                }
+                ch=uart_getchar(1);
+        }
 	// user just pressed return or enter just 1 digit
 	if (ch == '\r' || ch == '\n'){
 		lineendcnt--;
@@ -656,42 +675,42 @@ void terminalmode_next_line(void)
 
 void terminalmode(unsigned char chr_nl)
 {
-	signed char i;
+	unsigned char i;
 	// Init terminal
 	uart_sendstr_p(terminal_init);
 	//
-	uart_sendstr_P("AvrUsb500-1.4 terminal mode\r\n");
-	uart_sendstr_P("Enter SW Version Major [");
-	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 10);
+	uart_sendstr_P("AvrUsb500-1.5 terminal mode\r\n");
+	uart_sendstr_P("Enter SW Version Major in hex [");
+	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 16);
 	uart_sendstr((char *)msg_buf);
 	uart_sendstr_P("]: ");
 	i=terminalmode_readnum(chr_nl);
-	if (i >=0){
+	if (i!=0xff){
 		CONFIG_PARAM_SW_MAJOR=(unsigned char)i;
 	}
 	terminalmode_next_line();
-	uart_sendstr_P("Enter SW Version Minor [");
-	utoa(CONFIG_PARAM_SW_MINOR,(char *)msg_buf, 10);
+	uart_sendstr_P("Enter SW Version Minor in hex [");
+	utoa(CONFIG_PARAM_SW_MINOR,(char *)msg_buf, 16);
 	uart_sendstr((char *)msg_buf);
 	uart_sendstr_P("]: ");
 	i=terminalmode_readnum(chr_nl);
-	if (i >=0){
-		CONFIG_PARAM_SW_MINOR=(unsigned char)i;
+	if (i!=0xff){
+		CONFIG_PARAM_SW_MINOR=i;
 	}
 	terminalmode_next_line();
 	eeprom_write_byte(EEPROM_MINOR, CONFIG_PARAM_SW_MINOR);
 	eeprom_write_byte(EEPROM_MAJOR, CONFIG_PARAM_SW_MAJOR);
 	eeprom_write_byte(EEPROM_MAGIC, 20);// magic number=data is valid
 	uart_sendstr_P("\r\nOK, my SW version is now: ");
-	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 10);
+	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 16);
 	uart_sendstr((char *)msg_buf);
 	uart_sendchar('.');
-	if (CONFIG_PARAM_SW_MINOR < 10){
+	if (CONFIG_PARAM_SW_MINOR < 16){
 		uart_sendchar('0');
 	}
-	utoa(CONFIG_PARAM_SW_MINOR,(char *)msg_buf, 10);
+	utoa(CONFIG_PARAM_SW_MINOR,(char *)msg_buf, 16);
 	uart_sendstr((char *)msg_buf);
-	uart_sendstr_P("\r\n");
+	uart_sendstr_P(" (hex)\r\n");
 	uart_sendstr_P("Ready. Just close the terminal. No reset needed.\r\n");
 }
 
