@@ -9,7 +9,7 @@
 #include "timeout.h"
 // timing for software spi:
 #define F_CPU 3686400UL  // 3.6864 MHz
-#include <avr/delay.h>
+#include <util/delay.h>
 static unsigned char sck_dur=1;
 static unsigned char spi_in_sw=0;
 //static unsigned char sck_dur=12;
@@ -74,35 +74,55 @@ unsigned char spi_get_sck_duration(void)
         return(sck_dur);
 }
 
+void spi_sck_pulse(void)
+{
+        uint8_t spcrval;
+        spcrval=SPCR; // store old value
+        SPCR=0x00;     // SPI off
+        cbi(PORTB,PB5); // SCK low
+        _delay_ms(0.10);
+        sbi(PORTB,PB5); // SCK high
+        _delay_ms(0.10);
+        cbi(PORTB,PB5); // SCK low
+        SPCR=spcrval;
+}
+
+void spi_reset_pulse(void) 
+{
+        /* give a positive RESET pulse, we can't guarantee
+         * that SCK was low during power up (see Atmel's
+         * data sheets, search for "Serial Downloading in e.g atmega8 
+         * data sheet):
+         * the programmer can not guarantee that SCK is held low during
+         * Power-up. In this case, RESET must be given a positive pulse of at least two
+         * CPU clock cycles duration after SCK has been set to 0."*/
+        PORTB|= (1<<PB2); // reset = high = not active
+        _delay_ms(0.10);
+        PORTB &= ~(1<<PB2); // reset = low, stay active
+        delay_ms(20); // min stab delay
+}
+
 void spi_init(void) 
 {
-        cbi(PORTB,PB3); // MOSI low
-        cbi(PORTB,PB5); // SCK low
-        //
-        sbi(DDRB,PB3); // MOSI is output
-        sbi(DDRB,PB5); // SCK is output
-	// now output pins low in case somebody used it as output in his/her circuit
-        cbi(PORTB,PB3); // MOSI low
-        cbi(PORTB,PB5); // SCK low
-	//
-        cbi(DDRB,PINB4); // MISO is input
-        //
         sbi(DDRB,PB2); // pb2 as reset pin is output, this is also the ss pin which
                        // must be output for master to work.
         sbi(PORTB,PB2); // reset = high = not active
-        delay_ms(25); // discharge MOSI/SCK
+        //
+	// now output pins low in case somebody used it as output in his/her circuit
+        sbi(DDRB,PB3); // MOSI is output
+        cbi(PORTB,PB3); // MOSI low
+        sbi(DDRB,PB5); // SCK is output
+        cbi(PORTB,PB5); // SCK low
+        delay_ms(20); // discharge MOSI/SCK
         if (spi_in_sw==0){
                 /* Enable SPI, Master, set clock rate fck/16 */
                 SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
-        }
-        //
-        cbi(PORTB,PB2); // reset = low, stay active
-        cbi(PORTB,PB3); // MOSI low
-        cbi(PORTB,PB5); // SCK low
-        delay_ms(25); // minimum stab.-delay
-        if(spi_in_sw==0){
                 spi_set_sck_duration(sck_dur);
+                // now SCK and MOSI are under control of SPI
         }
+        cbi(PORTB,PB2); // reset = low, stay active
+        delay_ms(20); // stab delay
+        spi_reset_pulse();
 }
 
 // send 8 bit, return received byte
