@@ -19,7 +19,7 @@ int main(int argc, char *argv[])
         int fd;
 
         if (argc != 2){
-                printf("USAGE: ttydevinit /dev/usb/tts/0\n");
+                printf("USAGE: ttydevinit1152 /dev/ttyUSB0\n");
                 exit(0);
         }
         device=argv[1];
